@import "variables.css";

.jp-VoilaIcon {
  background-image: var(--jp-icon-voila);
}

.voila-viewWidget {
  display: block;
  height: 100%;

  min-width: 240px;
  min-height: 120px;

  padding: var(--jp-notebook-padding);
  outline: none;
  overflow: auto;
  background: var(--jp-layout-color0);
}

.voila-viewWidget.jp-mod-scrollPastEnd::after {
  display: block;
  content: '';
  min-height: var(--jp-notebook-scroll-padding);
}

.jp-LabShell #jp-top-panel {
  min-height: initial;
}


/*-----------------------------------------------------------------------------
| Presentation Mode (.jp-mod-presentationMode)
|----------------------------------------------------------------------------*/

.jp-mod-presentationMode .voila-viewWidget {
  --jp-content-font-size1: var(--jp-content-presentation-font-size1);
  --jp-code-font-size: var(--jp-code-presentation-font-size);
}
